@@ -1001,6 +1001,21 @@ class FunctionCallErrorBuilder:
         return builder
 
 
+def build_diagnostic_from_message(
+    message: str, code: str, source_line_start: int, source_line_end: int
+) -> DiagnosticGQL:
+    return DiagnosticGQL(
+        message=message,
+        range=RangeGQL(
+            start=PositionGQL(line=source_line_start, character=0),
+            end=PositionGQL(line=source_line_end + 1, character=0),
+        ),
+        severity=DiagnosticSeverityGQL.Error,
+        code=code,
+        codeDescription=None,
+    )
+
+
 """
 The following helper methods are empirically proven to be correct.
 The lines are 0 indexed and inclusive.
