@@ -8,7 +8,7 @@ import random
 import typing
 import warnings
 from functools import cached_property
-from typing import TYPE_CHECKING, Any, Callable, List, Literal, Mapping, Optional, Sequence, Tuple, TypeVar, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Literal, Mapping, Optional, Sequence, Tuple, TypeVar, Union
 from urllib.parse import urlparse
 
 import grpc
@@ -18,8 +18,9 @@ from google.protobuf import empty_pb2, timestamp_pb2
 from chalk import DataFrame, EnvironmentId, chalk_logger
 from chalk._gen.chalk.auth.v1.agent_pb2 import CustomClaim
 from chalk._gen.chalk.auth.v1.permissions_pb2 import Permission
-from chalk._gen.chalk.common.v1 import offline_query_pb2, online_query_pb2, upload_features_pb2
+from chalk._gen.chalk.common.v1 import online_query_pb2, resources_pb2, upload_features_pb2
 from chalk._gen.chalk.common.v1.online_query_pb2 import GenericSingleQuery, UploadFeaturesBulkRequest
+from chalk._gen.chalk.common.v1.script_task_pb2 import ScriptTaskKind, ScriptTaskRequest, TrainingRunArgs
 from chalk._gen.chalk.common.v2.execute_plan_pb2 import ExecutePlanRequest, ExecutePlanResponse
 from chalk._gen.chalk.engine.v1 import query_server_pb2
 from chalk._gen.chalk.engine.v1.query_server_pb2_grpc import QueryServiceStub
@@ -36,6 +37,11 @@ from chalk._gen.chalk.protosql.v1.sql_service_pb2 import (
 )
 from chalk._gen.chalk.protosql.v1.sql_service_pb2_grpc import SqlServiceStub
 from chalk._gen.chalk.server.v1.auth_pb2_grpc import AuthServiceStub
+from chalk._gen.chalk.server.v1.dataplanejobqueue_pb2 import (
+    GetJobQueueOperationSummaryRequest,
+    GetJobQueueOperationSummaryResponse,
+)
+from chalk._gen.chalk.server.v1.dataplanejobqueue_pb2_grpc import DataPlaneJobQueueServiceStub
 from chalk._gen.chalk.server.v1.deploy_pb2 import (
     CreateBranchFromSourceDeploymentRequest,
     CreateBranchFromSourceDeploymentResponse,
@@ -49,9 +55,14 @@ from chalk._gen.chalk.server.v1.graph_pb2 import (
     PythonVersion,
 )
 from chalk._gen.chalk.server.v1.graph_pb2_grpc import GraphServiceStub
+from chalk._gen.chalk.server.v1.log_pb2_grpc import LogSearchServiceStub
 from chalk._gen.chalk.server.v1.model_registry_pb2 import (
+    CreateModelArtifactRequest,
+    CreateModelArtifactResponse,
     CreateModelRequest,
     CreateModelResponse,
+    CreateModelVersionFromArtifactRequest,
+    CreateModelVersionFromArtifactResponse,
     CreateModelVersionRequest,
     CreateModelVersionResponse,
     GetModelArtifactUploadUrlsRequest,
@@ -62,8 +73,9 @@ from chalk._gen.chalk.server.v1.model_registry_pb2 import (
     GetModelVersionResponse,
 )
 from chalk._gen.chalk.server.v1.model_registry_pb2_grpc import ModelRegistryServiceStub
-from chalk._gen.chalk.server.v1.offline_queries_pb2 import CreateModelTrainingJobRequest, CreateModelTrainingJobResponse
 from chalk._gen.chalk.server.v1.offline_queries_pb2_grpc import OfflineQueryMetadataServiceStub
+from chalk._gen.chalk.server.v1.script_tasks_pb2 import CreateScriptTaskRequest, CreateScriptTaskResponse
+from chalk._gen.chalk.server.v1.script_tasks_pb2_grpc import ScriptTaskServiceStub
 from chalk._gen.chalk.server.v1.team_pb2 import (
     CreateServiceTokenRequest,
     CreateServiceTokenResponse,
@@ -83,6 +95,7 @@ from chalk.client.models import (
     ModelUploadUrlResponse,
     OnlineQuery,
     OnlineQueryResponse,
+    RegisterModelArtifactResponse,
     RegisterModelResponse,
     RegisterModelVersionResponse,
     ResourceRequests,
@@ -98,7 +111,7 @@ from chalk.features._encoding.outputs import encode_outputs
 from chalk.features.feature_set import is_feature_set_class
 from chalk.features.tag import DeploymentId
 from chalk.importer import CHALK_IMPORT_FLAG
-from chalk.ml import ModelEncoding, ModelType, SourceConfig
+from chalk.ml import LocalSourceConfig, ModelEncoding, ModelRunCriterion, ModelType, SourceConfig
 from chalk.ml.model_file_transfer import ModelFileUploader
 from chalk.parsed._proto.utils import datetime_to_proto_timestamp, value_to_proto
 from chalk.utils import df_utils
@@ -275,6 +288,12 @@ class StubProvider:
             raise RuntimeError("Unable to connect to API server.")
         return ModelRegistryServiceStub(self._server_channel)
 
+    @cached_property
+    def task_stub(self) -> ScriptTaskServiceStub:
+        if self._server_channel is None:
+            raise RuntimeError("Unable to connect to API server.")
+        return ScriptTaskServiceStub(self._server_channel)
+
     @cached_property
     def builder_stub(self) -> "BuilderServiceStub":
         from chalk._gen.chalk.server.v1.builder_pb2_grpc import BuilderServiceStub
@@ -283,6 +302,18 @@ class StubProvider:
             raise RuntimeError("Unable to connect to API server.")
         return BuilderServiceStub(self._server_channel)
 
+    @cached_property
+    def log_stub(self) -> LogSearchServiceStub:
+        if self._server_channel is None:
+            raise RuntimeError("Unable to connect to API server.")
+        return LogSearchServiceStub(self._server_channel)
+
+    @cached_property
+    def job_queue_stub(self) -> DataPlaneJobQueueServiceStub:
+        if self._server_channel is None:
+            raise RuntimeError("Unable to connect to API server.")
+        return DataPlaneJobQueueServiceStub(self._server_channel)
+
     def __init__(
         self,
         token_config: TokenConfig,
@@ -477,9 +508,22 @@ class StubRefresher:
     def call_model_stub(self, fn: Callable[[ModelRegistryServiceStub], T]) -> T:
         return self._retry_callable(fn, lambda: self._stub.model_stub)
 
+    def call_task_stub(self, fn: Callable[[ScriptTaskServiceStub], T]) -> T:
+        return self._retry_callable(fn, lambda: self._stub.task_stub)
+
     def call_builder_stub(self, fn: Callable[["BuilderServiceStub"], T]) -> T:
         return self._retry_callable(fn, lambda: self._stub.builder_stub)
 
+    def call_log_stub(self, fn: Callable[[LogSearchServiceStub], T]) -> T:
+        return self._retry_callable(fn, lambda: self._stub.log_stub)
+
+    def call_job_queue_stub(self, fn: Callable[[DataPlaneJobQueueServiceStub], T]) -> T:
+        return self._retry_callable(fn, lambda: self._stub.job_queue_stub)
+
+    @property
+    def log_stub(self) -> LogSearchServiceStub:
+        return self._stub.log_stub
+
     @property
     def environment_id(self) -> str | None:
         return self._stub.environment_id
@@ -1297,7 +1341,6 @@ class ChalkGRPCClient:
                 return GetRegisteredModelVersionResponse(
                     model_id=model_version_resp.model_version.id,
                     model_name=model_version_resp.model_version.model_name,
-                    metadata=dict(model_version_resp.model_version.metadata),
                     created_by=model_version_resp.model_version.created_by,
                     created_at=model_version_resp.model_version.created_at.ToDatetime(),
                     model_artifact=model_version_resp.model_version.model_artifact,
@@ -1327,14 +1370,14 @@ class ChalkGRPCClient:
             except grpc.RpcError as e:
                 raise RuntimeError(f"Could not register model version. {e.details()}")
 
-    def register_model(
+    def register_model_namespace(
         self,
         name: str,
         description: str,
-        metadata: Mapping[str, Any],
+        metadata: Optional[Mapping[str, Any]] = None,
     ) -> RegisterModelResponse:
         """
-        Register a model in the Chalk model registry.
+        Register a model namespace in the Chalk model registry.
 
         Parameters
         ----------
@@ -1342,7 +1385,7 @@ class ChalkGRPCClient:
             Unique name for the model
         description : str
             Description of the model's purpose and functionality
-        metadata : Mapping[str, Any]
+        metadata : Mapping[str, Any], optional
             Additional metadata dictionary containing framework info,
             training details, performance metrics, etc.
 
@@ -1357,7 +1400,7 @@ class ChalkGRPCClient:
 
         >>> from chalk.client import ChalkClient
         >>> client = ChalkClient()
-        >>> client.register_model(
+        >>> client.register_model_namespace(
         ...     name="RiskModel",
         ...     description="Credit risk assessment model using transaction history",
         ...     metadata={
@@ -1405,12 +1448,12 @@ class ChalkGRPCClient:
     def register_model_version(
         self,
         name: str,
-        model_type: ModelType,
+        model_type: Optional[ModelType] = None,
         model_encoding: Optional[ModelEncoding] = None,
         aliases: Optional[List[str]] = None,
         model: Optional[Any] = None,
         model_paths: Optional[List[str]] = None,
-        additional_files: Optional[Mapping[str, str]] = None,
+        additional_files: Optional[List[str]] = None,
         input_schema: Optional[Any] = None,
         output_schema: Optional[Any] = None,
         metadata: Optional[Mapping[str, Any]] = None,
@@ -1428,13 +1471,13 @@ class ChalkGRPCClient:
            Unique name for the model
         aliases : list of str, optional
            List of version aliases (e.g., ["v1.0", "latest"])
-        model_paths : list of str, optional
-           Paths to model files (for file-based registration)
         model : object, optional
            Python model object (for object-based registration)
-        additional_files : list of str, optional
+        model_paths : list of str, optional
+           Paths to model files (for file-based registration)
+        additional_files : List[str], optional
            Additional files needed for inference (tokenizers, configs, etc.)
-        model_type : ModelType
+        model_type : ModelType, optional
            Type of model framework
         model_encoding : ModelEncoding, optional
            Serialization format
@@ -1500,6 +1543,9 @@ class ChalkGRPCClient:
         with ModelSerializer.from_model(model, model_type) as model_serializer:
             model_file_uploader = ModelFileUploader(source_config)
 
+            if model_type is None:
+                model_type = model_serializer.model_type
+
             if model is not None:
                 inferred_input_schema, inferred_output_schema = model_serializer.infer_input_output_schemas(
                     model, model_type
@@ -1518,14 +1564,18 @@ class ChalkGRPCClient:
                 raise ValueError("You must specify an output_schema to register a model version.")
 
             try:
+                dir_allowlist: List[str] = []
                 metadata_converted = model_serializer.convert_metadata_to_protobuf(metadata)
 
-                tmp_path = ""
                 if model_paths is None:
                     if model is None:
                         raise ValueError("Failed to register model. Please specify a model or model_paths.")
+
+                    assert model_type is not None, "Unable to determine Model Type, please set parameter model_type."
+
                     tmp_path, model_encoding = model_serializer.serialize_model(model, model_type)
                     model_paths = [tmp_path]
+                    dir_allowlist.append(tmp_path)
                 else:
                     if model is not None:
                         raise ValueError(
@@ -1551,7 +1601,7 @@ class ChalkGRPCClient:
                     all_files_to_process,
                     model_file_names,
                     presigned_s3_response.upload_urls,
-                    [tmp_path],
+                    dir_allowlist,
                 )
                 try:
                     resp: CreateModelVersionResponse = self._stub_refresher.call_model_stub(
@@ -1588,7 +1638,6 @@ class ChalkGRPCClient:
                         model_version=resp.model_version.version,
                         artifact=resp.model_version.model_artifact,
                         aliases=list(resp.model_version.aliases),
-                        metadata=dict(resp.model_version.metadata),
                         created_by=resp.model_version.created_by,
                         created_at=resp.model_version.created_at.ToDatetime(),
                     )
@@ -1596,23 +1645,196 @@ class ChalkGRPCClient:
                     raise RuntimeError(f"Could not register model version. {e.details()}")
             except Exception as e:
                 raise RuntimeError(f"Could not register model version. {e}")
-        raise RuntimeError("Error creating model serializer context.")
+        raise RuntimeError("Error creating model serializer context to register model version.")
+
+    def _upload_model_artifact(
+        self,
+        model: Any,
+        additional_files: Optional[List[str]] = None,
+        metadata: Optional[Dict[str, Any]] = None,
+    ) -> RegisterModelArtifactResponse:
+        with ModelSerializer.from_model(model) as model_serializer:
+            try:
+                dir_allowlist: List[str] = []
+                model_tmp_path, model_encoding = model_serializer.serialize()
+                dir_allowlist.append(model_tmp_path)
+
+                model_file_uploader = ModelFileUploader(LocalSourceConfig())
+
+                all_files_to_process, model_file_names = model_file_uploader.prepare_file_mapping(
+                    [model_tmp_path], additional_files
+                )
+
+                presigned_s3_response: ModelUploadUrlResponse = self._get_model_artifact_presigned_s3(
+                    model_paths=list(all_files_to_process.keys())
+                )
+
+                metadata_converted = model_serializer.convert_metadata_to_protobuf(metadata)
+
+                input_schema, output_schema = model_serializer.infer_input_output_schemas()
+
+                input_model_schema = model_serializer.convert_schema(input_schema)
+                output_model_schema = model_serializer.convert_schema(output_schema)
+
+                model_upload_paths, additional_files_upload_paths = model_file_uploader.upload_files(
+                    file_paths=all_files_to_process,
+                    model_file_names=model_file_names,
+                    presigned_urls=presigned_s3_response.upload_urls,
+                    dir_allowlist=dir_allowlist,
+                )
+
+                dependencies = model_serializer.get_dependencies()
+                try:
+                    resp: CreateModelArtifactResponse = self._stub_refresher.call_model_stub(
+                        lambda x: x.CreateModelArtifact(
+                            CreateModelArtifactRequest(
+                                model_artifact_id=presigned_s3_response.model_artifact_id,
+                                model_artifact=_model_artifact_pb2.ModelArtifactSpec(
+                                    model_files=[
+                                        model_serializer.fileinfo_to_protobuf(file) for file in model_upload_paths
+                                    ],
+                                    additional_files=[
+                                        model_serializer.fileinfo_to_protobuf(file)
+                                        for file in additional_files_upload_paths
+                                    ],
+                                    model_type=model_serializer.model_type,
+                                    model_encoding=model_encoding,
+                                    model_signature=_model_artifact_pb2.ModelSignature(
+                                        inputs=input_model_schema,
+                                        outputs=output_model_schema,
+                                    ),
+                                    input_features=[],
+                                    output_features=[],
+                                    python_dependencies=dependencies,
+                                ),
+                                metadata=metadata_converted,
+                            )
+                        )
+                    )
+                    return RegisterModelArtifactResponse(
+                        artifact_id=resp.model_artifact.id,
+                        path=resp.model_artifact.path,
+                        spec=resp.model_artifact.spec,
+                        metadata=dict(resp.model_artifact.metadata),
+                        created_by=resp.model_artifact.created_by,
+                        created_at=resp.model_artifact.created_at.ToDatetime(),
+                    )
+                except grpc.RpcError as e:
+                    raise RuntimeError(f"Could not register model artifact. {e.details()}")
+            except Exception as e:
+                raise RuntimeError(f"Could not register model artifact. {e}")
+        raise RuntimeError("Error creating model serializer context to create model artifact.")
+
+    def promote_model_artifact(
+        self,
+        name: str,
+        model_artifact_id: Optional[str] = None,
+        run_id: Optional[str] = None,
+        criterion: Optional[ModelRunCriterion] = None,
+        aliases: Optional[List[str]] = None,
+    ) -> RegisterModelVersionResponse:
+        """
+        Register a model in the Chalk model registry.
+
+        Parameters
+        ----------
+        name : str
+            Name of the model namespace to promote into.
+        model_artifact_id: str, optional
+            Artifact UUID to promote to a model version.
+        run_id: str, optional
+            run id that produce the artifact to promote.
+        criterion: ModelRunCriterion, optional
+            criterion on which to select the artifact from the training run.
+            If none provided, the latest artifact in the run will be selected.
+        aliases: list of str, optional
+            List of version aliases (e.g., ["v1.0", "latest"])
+
+        Example
+        --------
+        Register from Python object:
+
+        >>> client.promote_model_artifact(
+        ...     name="RiskModel",
+        ...     model_artifact_id=model_artifact_id,
+        ...     aliases=["latest"],
+        ... )
+        """
+        if model_artifact_id is not None:
+            if run_id is not None or criterion is not None:
+                raise ValueError("Please specify only one of 'model_artifact_id' or (run_name, run criterion)")
+        else:
+            if run_id is None and criterion is None:
+                raise ValueError("Please specify at least one of 'model_artifact_id' or (run_name, run criterion)")
+
+        try:
+            resp: CreateModelVersionFromArtifactResponse = self._stub_refresher.call_model_stub(
+                lambda x: x.CreateModelVersionFromArtifact(
+                    CreateModelVersionFromArtifactRequest(
+                        model_name=name,
+                        model_artifact_id=model_artifact_id,
+                        training_run=ModelSerializer.convert_run_criterion_to_proto(
+                            run_name=run_id,
+                            criterion=criterion,
+                        ),
+                        aliases=aliases,
+                    )
+                )
+            )
+            return RegisterModelVersionResponse(
+                model_id=resp.model_version.id,
+                model_name=resp.model_version.model_name,
+                model_version=resp.model_version.version,
+                artifact=resp.model_version.model_artifact,
+                aliases=list(resp.model_version.aliases),
+                created_by=resp.model_version.created_by,
+                created_at=resp.model_version.created_at.ToDatetime(),
+            )
+        except grpc.RpcError as e:
+            raise RuntimeError(f"Could not promote model artifact. {e.details()}")
 
     def create_model_training_job(
         self,
-        train_fn: Callable[[Optional[Mapping[str, Any]]], bool],
-        model_name: str,
-        dataset_name: str,
-        config: Optional[Mapping[str, Any]] = None,
+        script: str,
+        function_name: str,
+        experiment_name: str,
+        branch: Optional[str] = None,
+        config: str | None = None,
         resources: Optional[ResourceRequests] = None,
-    ) -> CreateModelTrainingJobResponse:
-        return self._stub_refresher.call_offline_query_stub(
-            lambda x: x.CreateModelTrainingJob(
-                CreateModelTrainingJobRequest(
-                    training_job_request=offline_query_pb2.OfflineQueryRequest(
-                        dataset_name=dataset_name,
-                    )
-                )
+        env_overrides: Optional[Mapping[str, str]] = None,
+        enable_profiling: bool = False,
+        max_retries: int = 0,
+    ) -> CreateScriptTaskResponse:
+        resources_request = {}
+        if resources is not None:
+            if resources.cpu is not None:
+                resources_request["cpu"] = resources.cpu
+            if resources.memory is not None:
+                resources_request["memory"] = resources.memory
+
+        return self._stub_refresher.call_task_stub(
+            lambda x: x.CreateScriptTask(
+                CreateScriptTaskRequest(
+                    request=ScriptTaskRequest(
+                        function_reference_type="file",
+                        # Hardcoded script name
+                        function_reference=f"train.py::{function_name}",
+                        kind=ScriptTaskKind.SCRIPT_TASK_KIND_TRAINING_RUN,
+                        training_run=TrainingRunArgs(
+                            experiment_name=experiment_name,
+                        ),
+                        arguments_json=config,
+                        branch=branch,
+                        resource_requests=resources_pb2.ResourceRequirements(
+                            requests=resources_request,
+                        ),
+                        resource_group=resources.resource_group if resources is not None else None,
+                        env_overrides=env_overrides,
+                        enable_profiling=enable_profiling,
+                        max_retries=max_retries,
+                    ),
+                    source_file=script.encode("utf-8"),
+                ),
             )
         )
 
@@ -1664,3 +1886,100 @@ class ChalkGRPCClient:
                     f"Branch server did not start within {timeout_seconds} seconds. Last state: {BranchScalingState.Name(response.state)}"
                 )
             time.sleep(poll_interval_seconds)
+
+    def get_job_queue_operation_summary(
+        self,
+        operation_id: str,
+        environment_id: str | None = None,
+        limit: int | None = None,
+        offset: int | None = None,
+    ) -> GetJobQueueOperationSummaryResponse:
+        """Get summary information for a job queue operation.
+
+        Parameters
+        ----------
+        operation_id
+            The ID of the operation to get summary for
+        environment_id
+            The environment ID. If None, uses the client's environment.
+        limit
+            Maximum number of job rows to return. Defaults to 10000.
+        offset
+            Offset for pagination. Defaults to 0.
+
+        Returns
+        -------
+        GetJobQueueOperationSummaryResponse
+            The operation summary response containing job queue information.
+
+        Examples
+        --------
+        >>> from chalk.client.client_grpc import ChalkGRPCClient
+        >>> client = ChalkGRPCClient()
+        >>> response = client.get_job_queue_operation_summary(operation_id="op_123")
+        """
+        env_id = environment_id or self._stub_refresher.environment_id
+        if not env_id:
+            raise ValueError("No environment specified")
+
+        request = GetJobQueueOperationSummaryRequest(
+            operation_id=operation_id,
+            environment_id=env_id,
+        )
+
+        if limit is not None:
+            request.limit = limit
+        if offset is not None:
+            request.offset = offset
+
+        return self._stub_refresher.call_job_queue_stub(lambda x: x.GetJobQueueOperationSummary(request))
+
+    def follow_model_training_job(
+        self,
+        operation_id: str,
+        poll_interval: float = 2.0,
+        output_callback: Optional[Callable[[str, str], None]] = None,
+    ) -> None:
+        """Follow a model training job, displaying both status and logs.
+
+        This method polls the job queue for status updates while also following logs
+        in real-time. It continues until the job reaches a terminal state (completed,
+        failed, or canceled).
+
+        Parameters
+        ----------
+        operation_id
+            The operation ID of the model training job
+        poll_interval
+            Time in seconds between polling for status and logs. Defaults to 2.0 seconds.
+        output_callback
+            Optional callback function that receives (timestamp, message) for each log entry.
+            If None, logs are displayed using Rich live display.
+
+        Examples
+        --------
+        >>> from chalk.client.client_grpc import ChalkGRPCClient
+        >>> client = ChalkGRPCClient()
+        >>> client.follow_model_training_job(operation_id="op_123")
+        """
+        from chalk.utils.job_log_display import JobLogDisplay
+
+        # Create display manager
+        display = JobLogDisplay(title="Model Training Jobs")
+
+        # Define callback for status polling
+        def get_status_callback():
+            return self.get_job_queue_operation_summary(operation_id=operation_id)
+
+        # Get log stub and construct log query
+        log_query = f'operation_id:"{operation_id}"'
+        log_stub = self._stub_refresher.log_stub
+
+        # Delegate to the display manager to handle all threading and coordination
+        display.follow_job(
+            get_status_callback=get_status_callback,
+            log_stub=log_stub,
+            log_query=log_query,
+            poll_interval=poll_interval,
+            output_callback=output_callback,
+        )
