@@ -1325,7 +1325,7 @@ class DatasetRevisionImpl(DatasetRevision):
             actual_args: List[Any] = []
             for i, input in enumerate(resolver.inputs):
                 contains_packed_df_name = f"{i}_packed_df" in no_ts_input_df.columns
-                if unwrap_feature(input).is_has_many or contains_packed_df_name:
+                if contains_packed_df_name or unwrap_feature(input).is_has_many:
                     # bogus indexing scheme
                     col_name = (
                         f"{i}_packed_df" if contains_packed_df_name else f"{i}"
@@ -1363,6 +1363,13 @@ This occurred during the actual execution of resolver {resolver.fqn}.
                     )
                     raise e
             print(f"resolver_replay: {resolver.fqn} returned {output}")
+            if isinstance(output, DataFrame):
+                try:
+                    output = output.to_polars().collect().rows(named=True)
+                except Exception as e:
+                    raise RuntimeError(
+                        f'Failed to convert DataFrame output from resolver "{resolver.fqn}" during resolver replay'
+                    ) from e
             output_col.append(output)
         return raw_input_df.with_columns(pl.Series(name="__resolver_replay_output__", values=output_col))
 
