@@ -9,7 +9,6 @@ import re
 import weakref
 from collections.abc import Mapping, MutableMapping
 from datetime import timedelta, datetime
-from enum import Enum
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -68,6 +67,13 @@ _TPrim = TypeVar("_TPrim", bound=TPrimitive)
 
 _logger = get_logger(__name__)
 
+from chalk.features.feature_cache_strategy import (
+    CacheStrategy,
+    CacheNullsType,
+    CacheDefaultsType,
+    get_cache_strategy_from_cache_settings,
+)
+
 
 @dataclasses.dataclass(frozen=True)
 class HasOnePathObj:
@@ -189,16 +195,6 @@ class WindowConfigResolved:
     continuous_buffer_duration_seconds: int | None
 
 
-class CacheStrategy(Enum):
-    ALL = "all"
-    NO_NULLS = "no_nulls"
-    NO_DEFAULTS = "no_defaults"
-    NO_NULLS_OR_DEFAULTS = "no_nulls_or_defaults"
-    EVICT_NULLS = "evict_nulls"
-    EVICT_DEFAULTS = "evict_defaults"
-    EVICT_NULLS_AND_DEFAULTS = "evict_nulls_and_defaults"
-
-
 class Feature(Generic[_TPrim, _TRich]):
     window_materialization: MaterializationWindowConfig | Literal[True] | None
 
@@ -282,7 +278,7 @@ class Feature(Generic[_TPrim, _TRich]):
         underscore_expression: Underscore | None = None,
         max_staleness: Duration | None | ellipsis = ...,
         online_store_max_items: int | None = None,
-        cache_strategy: CacheStrategy = CacheStrategy.ALL,
+        cache_strategy: CacheStrategy = CacheStrategy.ALL_WITH_BOTH_UNSET,
         etl_offline_to_online: bool | None = None,
         encoder: TEncoder[_TPrim, _TRich] | None = None,
         decoder: TDecoder[_TPrim, _TRich] | None = None,
@@ -1231,6 +1227,7 @@ class Feature(Generic[_TPrim, _TRich]):
                 etl_offline_to_online=False,
                 default=None,
                 max_staleness=CHALK_MAX_TIMEDELTA,
+                cache_strategy=CacheStrategy.ALL,
                 encoder=self._encoder,  # pyright: ignore[reportArgumentType]
                 decoder=self._decoder,
                 pyarrow_dtype=self._pyarrow_dtype,
@@ -1668,6 +1665,7 @@ class Feature(Generic[_TPrim, _TRich]):
                     no_display=True,
                     is_autogenerated=True,
                     max_staleness=None,
+                    cache_strategy=CacheStrategy.ALL,
                     version=self.version and self.version.maximum,
                     default_version=(self.version and self.version.default) or 1,
                     primary=False,
@@ -1735,8 +1733,8 @@ def feature(
     default_version: int = 1,
     primary: Optional[bool] = None,
     max_staleness: Optional[Union[ellipsis, Duration]] = ...,
-    cache_nulls: Union[bool, Literal["evict_nulls"]] = True,
-    cache_defaults: Union[bool, Literal["evict_defaults"]] = True,
+    cache_nulls: Optional[CacheNullsType] = None,
+    cache_defaults: Optional[CacheDefaultsType] = None,
     etl_offline_to_online: Optional[bool] = None,
     encoder: Optional[TEncoder[_TPrim, _TRich]] = None,
     decoder: Optional[TDecoder[_TPrim, _TRich]] = None,
@@ -2060,45 +2058,10 @@ def feature(
             if not isinstance(value, Feature):  # pyright: ignore[reportUnnecessaryIsInstance]
                 raise ValueError(f"When `versions` is provided, the values must be features, but `{value}` was given.")
 
-    if cache_nulls == "evict_nulls":
-        if cache_defaults == "evict_defaults":
-            cache_strategy = CacheStrategy.EVICT_NULLS_AND_DEFAULTS
-        elif cache_defaults == False:
-            raise ValueError(
-                'Cannot evict nulls and not cache defaults. Did you mean to set `cache_nulls=False` or `cache_defaults="evict_defaults"`?'
-            )
-        elif cache_defaults == True:
-            cache_strategy = CacheStrategy.EVICT_NULLS
-        else:
-            raise ValueError(
-                f'Expected value of cache_defaults to be True, False, or "evict_defaults". Received {cache_defaults}'
-            )
-    elif cache_nulls is False:
-        if cache_defaults == "evict_defaults":
-            raise ValueError(
-                'Cannot evict defaults and not cache nulls. Did you mean to set `cache_defaults=False` or `cache_nulls="evict_nulls"`?'
-            )
-        elif cache_defaults == False:
-            cache_strategy = CacheStrategy.NO_NULLS_OR_DEFAULTS
-        elif cache_defaults == True:
-            cache_strategy = CacheStrategy.NO_NULLS
-        else:
-            raise ValueError(
-                f'Expected value of cache_defaults to be True, False, or "evict_defaults". Received {cache_defaults}'
-            )
-    elif cache_nulls is True:
-        if cache_defaults == "evict_defaults":
-            cache_strategy = CacheStrategy.EVICT_DEFAULTS
-        elif cache_defaults == False:
-            cache_strategy = CacheStrategy.NO_DEFAULTS
-        elif cache_defaults == True:
-            cache_strategy = CacheStrategy.ALL
-        else:
-            raise ValueError(
-                f'Expected value of cache_defaults to be True, False, or "evict_defaults". Received {cache_defaults}'
-            )
-    else:
-        raise ValueError(f'Expected value of cache_nulls to be True, False, or "evict_nulls". Received {cache_nulls}')
+    cache_strategy = get_cache_strategy_from_cache_settings(
+        cache_nulls=cache_nulls,
+        cache_defaults=cache_defaults
+    )
 
     return cast(
         _TRich,
