@@ -383,10 +383,22 @@ def _init_fn(
         ts_feature_name = self.__chalk_ts__.attribute_name
 
         for k, v in kwargs.items():
-            cls_field = getattr(self.__class__, k, None)
-            if type(cls_field) is not FeatureWrapper and k not in additional_inits and k != ts_feature_name:
+            actual_key = k
+            if "@" in k:
+                base_name, version_str = k.rsplit("@", 1)
+                try:
+                    version = int(version_str)
+                    actual_key = f"{base_name}_v{version}"
+                    if not hasattr(self.__class__, actual_key):
+                        actual_key = base_name
+                except ValueError:
+                    pass
+
+            cls_field = getattr(self.__class__, actual_key, None)
+            if type(cls_field) is not FeatureWrapper and actual_key not in additional_inits and actual_key != ts_feature_name:
                 raise TypeError(f"{self.__class__.__name__}.__init__() got an unexpected keyword argument '{k}'")
-            setattr(self, k, v)
+
+            setattr(self, actual_key, v)
 
     _init.__name__ = "__init__"
     return _init
