@@ -18,6 +18,9 @@ if TYPE_CHECKING:
 
 T = TypeVar("T")
 
+class NearestNeighborException(ValueError):
+    ...
+
 
 class _MarkedUnderlyingFeature:
     __slots__ = ("_fn", "_source", "_debug_info")
@@ -325,23 +328,25 @@ class FeatureWrapper:
         return FeatureWrapper(_MarkedUnderlyingFeature(fn, ("__getattr__", item)))
 
     def is_near(self, item: Any, metric: Literal["l2", "cos", "ip"] = "l2") -> Filter:
+        if metric not in ("l2", "cos", "ip"):
+            raise NearestNeighborException(f"Invalid metric '{metric}'. Must be one of 'l2', 'cos', or 'ip'.")
         from chalk.features import Feature
         other = unwrap_feature(item)
         underlying = self._chalk_get_underlying()
         if not isinstance(underlying, Feature):
-            raise TypeError(f"Feature.is_near() is only supported on features. {underlying} is not a feature.")
+            raise NearestNeighborException(f"Feature.is_near() is only supported on features. {underlying} is not a feature.")
         self_vector = underlying.typ.as_vector()
         if self_vector is None:
-            raise TypeError(
+            raise NearestNeighborException(
                 f"Nearest neighbor relationships are only supported for vector features. Feature '{underlying.root_fqn}' is not a vector."
             )
         other_vector = other.typ.as_vector()
         if other_vector is None:
-            raise TypeError(
+            raise NearestNeighborException(
                 f"Nearest neighbor relationships are only supported for vector features. Feature '{other.root_fqn}' is not a vector."
             )
         if underlying.converter.pyarrow_dtype != other.converter.pyarrow_dtype:
-            raise TypeError(
+            raise NearestNeighborException(
                 (
                     f"Nearest neighbor relationships are only supported if both vectors have the same data type and dimensions. "
                     f"Feature '{underlying.root_fqn}' is of type `{underlying.converter.pyarrow_dtype}` "
