@@ -20,7 +20,7 @@ _hook_logger = get_logger("chalk.hook_logger")
 
 async def _run_all_hooks(environment: str, venv: Optional[str], hooks: Iterable["Hook"]) -> None:
     for hook in hooks:
-        if (hook.environment is None or environment in hook.environment) and (hook.venv is None or venv == hook.venv):
+        if (hook.environment is None or environment in hook.environment) and venv == hook.venv:
             start_time = time.perf_counter()  # Start timing
             try:
                 _hook_logger.info("Starting to run hook %s...", hook.fn.__name__)
