@@ -18,7 +18,7 @@ from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Tuple, Ty
 import pyarrow as pa
 
 from chalk._lsp.error_builder import DiagnosticBuilder, LSPErrorBuilder
-from chalk.features import Feature, Features, FeatureSetBase, Filter, unwrap_feature
+from chalk.features import Feature, Features, FeatureSetBase, Filter, Vector, unwrap_feature
 from chalk.features.feature_field import WindowConfigResolved
 from chalk.features.pseudofeatures import Now
 
@@ -101,6 +101,8 @@ supported_aggs = (
     "sum",
     "var",
     "var_sample",
+    "vector_sum",
+    "vector_mean",
 )
 
 
@@ -169,7 +171,16 @@ def _check_types(
         return
 
     joined_annotation = joined_feature.typ.parsed_annotation
-    if aggregation not in {"count", "approx_count_distinct", "approx_top_k", "min_by_n", "max_by_n", "array_agg"}:
+    if aggregation not in {
+        "count",
+        "approx_count_distinct",
+        "approx_top_k",
+        "min_by_n",
+        "max_by_n",
+        "array_agg",
+        "vector_sum",
+        "vector_mean",
+    }:
         _validate_types(
             annotation=joined_annotation,
             permitted_types=(int, float),
@@ -195,7 +206,6 @@ def _check_types(
             joined=False,
             feature_name=feature_name,
         )
-
     elif aggregation == "min" or aggregation == "max":
         if _get_underlying_type(this_annotation, feature_name) != _get_underlying_type(
             joined_annotation, joined_feature.name
@@ -710,6 +720,14 @@ def parse_windowed_materialization(f: Feature) -> WindowConfigResolved | None:
         aggregated_feature_name=aggregated_value,
     )
 
+    if aggregation == "sum" or aggregation == "mean":
+        try:
+            if issubclass(f.typ.parsed_annotation, Vector):
+                aggregation = f"vector_{aggregation}"
+        except TypeError:
+            # Not a class so not a Vector, skip
+            pass
+
     _check_types(
         feature_name=f.window_stem,
         aggregation=aggregation,
