@@ -2,7 +2,9 @@ import os
 from dataclasses import dataclass
 from enum import Enum
 from functools import cache
-from typing import Mapping, Tuple
+from typing import Literal, Mapping, Tuple
+
+import pyarrow as pa
 
 import chalk._gen.chalk.models.v1.model_artifact_pb2 as pb
 import chalk._gen.chalk.models.v1.model_version_pb2 as mv_pb
@@ -10,6 +12,16 @@ import chalk._gen.chalk.models.v1.model_version_pb2 as mv_pb
 REGISTRY_METADATA_FILE = os.getenv("CHALK_MODEL_REGISTRY_METADATA_FILENAME", None)
 CHALK_MODEL_REGISTRY_ROOT = os.getenv("CHALK_MODEL_REGISTRY_ROOT", "/models")
 
+MODEL_METADATA_PREFIX = "__chalk_model__"
+
+MODEL_TRAIN_METADATA_RUN_NAME = f"{MODEL_METADATA_PREFIX}run_name__"
+
+MODEL_TRAIN_RUN_NAME_ENV_VAR = "CHALK_MODEL_TRAIN_RUN_NAME"
+
+
+def get_model_metadata_run_name_from_env():
+    return os.getenv(MODEL_TRAIN_RUN_NAME_ENV_VAR, "")
+
 
 class ModelType(str, Enum):
     PYTORCH = "MODEL_TYPE_PYTORCH"
@@ -32,6 +44,12 @@ class ModelEncoding(str, Enum):
     SAFETENSOR = "MODEL_ENCODING_SAFETENSORS"
 
 
+@dataclass
+class ModelRunCriterion:
+    direction: Literal["max", "min"]
+    metric: str
+
+
 @dataclass
 class LoadedModel:
     spec: pb.ModelArtifactSpec
@@ -118,8 +136,6 @@ def model_encoding_from_proto(me: pb.ModelEncoding) -> ModelEncoding:
 
 from typing import Any, List, Optional, Tuple
 
-import pyarrow as pa
-
 
 class ModelAttributeExtractor:
     @staticmethod
@@ -293,3 +309,95 @@ class ModelAttributeExtractor:
             pass
 
         return input_schema, output_schema
+
+    @staticmethod
+    def infer_model_type(model: Any) -> Optional[ModelType]:
+        # PYTORCH
+        try:
+            import torch.nn as nn  # pyright: ignore[reportMissingImports]
+
+            if isinstance(model, nn.Module):
+                return ModelType.PYTORCH
+        except ImportError:
+            pass
+
+        # XGBOOST
+        try:
+            import xgboost as xgb  # pyright: ignore[reportMissingImports]
+
+            if isinstance(model, (xgb.XGBModel, xgb.Booster)):
+                return ModelType.XGBOOST
+            # Also check for XGBoost sklearn API
+            if hasattr(model, "__class__") and "xgboost" in model.__class__.__module__:
+                return ModelType.XGBOOST
+        except ImportError:
+            pass
+
+        # LIGHTGBM
+        try:
+            import lightgbm as lgb  # pyright: ignore[reportMissingImports]
+
+            if isinstance(model, (lgb.LGBMModel, lgb.Booster)):
+                return ModelType.LIGHTGBM
+            if hasattr(model, "__class__") and "lightgbm" in model.__class__.__module__:
+                return ModelType.LIGHTGBM
+        except ImportError:
+            pass
+
+        # CATBOOST
+        try:
+            import catboost as cb  # pyright: ignore[reportMissingImports]
+
+            # CatBoost has various model classes
+            if hasattr(model, "__class__") and "catboost" in model.__class__.__module__:
+                return ModelType.CATBOOST
+            # Common CatBoost classes
+            try:
+                if isinstance(model, (cb.CatBoost, cb.CatBoostClassifier, cb.CatBoostRegressor)):
+                    return ModelType.CATBOOST
+            except (AttributeError, NameError):
+                pass
+        except ImportError:
+            pass
+
+        # ONNX - do we need this one?
+        try:
+            import onnx  # pyright: ignore[reportMissingImports]
+
+            if isinstance(model, onnx.ModelProto):
+                return ModelType.ONNX
+        except ImportError:
+            pass
+
+        try:
+            import onnxruntime  # pyright: ignore[reportMissingImports]
+
+            if isinstance(model, onnxruntime.InferenceSession):
+                return ModelType.ONNX
+        except ImportError:
+            pass
+
+        # SKLEARN
+        try:
+            import sklearn.base  # pyright: ignore[reportMissingImports]
+
+            if isinstance(model, sklearn.base.BaseEstimator):
+                return ModelType.SKLEARN
+
+            if hasattr(model, "__class__") and "sklearn" in model.__class__.__module__:
+                return ModelType.SKLEARN
+        except ImportError:
+            pass
+
+        # TENSORFLOW
+        try:
+            import tensorflow as tf  # pyright: ignore[reportMissingImports]
+
+            if isinstance(model, tf.keras.Model):
+                return ModelType.TENSORFLOW
+            if hasattr(model, "__class__") and "tensorflow" in model.__class__.__module__:
+                return ModelType.TENSORFLOW
+        except ImportError:
+            pass
+
+        return None
