@@ -15,6 +15,7 @@ from chalk.features.resolver import Cron, OfflineResolver, OnlineResolver, SinkR
 from chalk.ml.model_reference import ModelReference
 from chalk.parsed.duplicate_input_gql import (
     FeatureClassGQL,
+    ModelRelationGQL,
     RecomputeFeaturesGQL,
     UpsertCDCSourceGQL,
     UpsertCronQueryGQL,
@@ -477,6 +478,11 @@ def convert_type_to_gql(
             version=t.version,
             asOf=t.as_of_date,
             alias=t.alias,
+            relations=[
+                ModelRelationGQL(inputFeatures=input_features, outputFeature=output_feature)
+                for input_features, output_feature in t.relations
+            ],
+            resolvers=t.resolvers,
             filename=t.filename if path_prefix is None else t.filename.replace(path_prefix, ""),
             code=t.code,
             sourceLineStart=t.source_line_start,
