@@ -26,7 +26,7 @@ if TYPE_CHECKING:
     from google.protobuf.message import Message as ProtobufMessage
 
     from chalk import Document, Windowed
-    from chalk.features import DataFrame, Feature, Features, Vector
+    from chalk.features import DataFrame, Feature, Features, Tensor, Vector
     from chalk.streams import Windowed
 
 _logger = get_logger(__name__)
@@ -52,6 +52,7 @@ class ParsedAnnotation:
         "_as_features_cls",
         "_as_dataframe",
         "_as_vector",
+        "_as_tensor",
         "_as_feature",
     )
 
@@ -90,6 +91,7 @@ class ParsedAnnotation:
         self._as_features_cls: Optional[Type[Features]] = None
         self._as_dataframe: Optional[Type[DataFrame]] = None
         self._as_vector: Optional[Type[Vector]] = None
+        self._as_tensor: Optional[Type[Tensor]] = None
         self._as_feature: Optional[Feature] = None
 
     @property
@@ -279,6 +281,7 @@ class ParsedAnnotation:
         raise TypeError(message)
 
     def _parse_type(self, annotation: Union[type, Windowed, Annotated]):
+        from chalk.features._tensor import Tensor
         from chalk.features._vector import Vector
         from chalk.features.dataframe import DataFrame
         from chalk.features.feature_field import Feature
@@ -496,6 +499,9 @@ class ParsedAnnotation:
             elif issubclass(self._underlying, Vector):
                 self._as_vector = self._underlying
 
+            elif issubclass(self._underlying, Tensor):
+                self._as_tensor = self._underlying
+
             elif issubclass(self._underlying, timedelta):
                 self._type_error(
                     (f"Invalid type annotation for feature '{error_ctx}': timedelta feature types are not supported."),
@@ -542,6 +548,11 @@ class ParsedAnnotation:
             self._parse_annotation()
         return self._as_vector
 
+    def as_tensor(self) -> Optional[Type[Tensor]]:
+        if self._parsed_annotation is None:
+            self._parse_annotation()
+        return self._as_tensor
+
     def as_feature(self) -> Optional[Feature]:
         if self._parsed_annotation is None:
             self._parse_annotation()
